`timescale 1ns / 1ps
//
//					  (C) Copyright 2007-2008 John B. Stephensen
//
// This Verilog source file and all its derivatives are licensed only for
// personal non-profit educational use in the Amateur Radio Service and
// the license is not transferrable. The information is provided as-is for
// experimental purposes and the author does not warranty its freedom
// from defects or its suitability for any specific application.
//
module T_CPU16;

	// Inputs
	reg [15:0] din;
	reg clk;
	reg rst;
	wire [15:0] min;
	reg [9:0] taddr;
	reg [15:0] tin;
	reg twr;
	reg tclk;

	// Outputs
	wire [13:0] maddr;
	wire mwr;
	wire [6:0] ioaddr;
	wire iord;
	wire [15:0] dout;
	wire iowr;
	wire [15:0] mout;
	wire [15:0] tout;

	// Instantiate the Unit Under Test (UUT)
	CPU16 uut (
		.ioaddr(ioaddr), 
		.din(din), 
		.iord(iord), 
		.dout(dout), 
		.iowr(iowr), 
		.clk(clk), 
		.rst(rst), 
		.maddr(maddr), 
		.min(min), 
		.mout(mout), 
		.mwr(mwr)
	);
	// instantiate dual-port memory
	RAMB16_S18_S18 ram (
		.ADDRA(maddr[9:0]),
		.DIA(mout),
		.DIPA(2'b11),
		.DOA(min),
		.DOPA(),
		.CLKA(clk),
		.WEA(mwr),
		.ENA(1'b1),
		.SSRA(1'b0),
		.ADDRB(taddr),
		.DIB(tin),
		.DIPB(2'b11),
		.DOB(tout),
		.DOPB(),
		.CLKB(tclk),
		.WEB(twr),
		.ENB(1'b1),
		.SSRB(1'b0)
		);

	initial begin
		// Initialize Inputs
		din = 16'h1234;
		clk = 0;
		rst = 0;
		taddr = 0;
		tin = 0;
		twr = 0;
		tclk = 0;

		// Wait 100 ns for global reset to finish
		#100;
        
		// load memory
		#8 tclk = 0; twr = 1; taddr = 1; tin = 1; 
		#8 tclk = 1; twr = 1; taddr = 1; tin = 1; 
		#8 tclk = 0; twr = 1; taddr = 2; tin = 4; 
		#8 tclk = 1; twr = 1; taddr = 2; tin = 4; 
		#8 tclk = 0; twr = 1; taddr = 3; tin = 9; 
		#8 tclk = 1; twr = 1; taddr = 3; tin = 9; 
		#8 tclk = 0; twr = 1; taddr = 4; tin = 16; 
		#8 tclk = 1; twr = 1; taddr = 4; tin = 16; 
		#8 tclk = 0; twr = 0; taddr = 0; tin = 0;
		// reset CPU
		#8 clk = 0; rst = 1;
		#8 clk = 1; rst = 1;
		#8 clk = 0; rst = 0;
		// execute instructions
		repeat (100)
		begin
			#8 clk = 0;
			#8 clk = 1;
		end
		#8 clk = 0;
		// read memory
		#8 tclk = 0; twr = 0; taddr = 0;
		#8 tclk = 1; twr = 0; taddr = 0;
		#8 tclk = 0; twr = 0; taddr = 1; 
		#8 tclk = 1; twr = 0; taddr = 1; 
		#8 tclk = 0; twr = 0; taddr = 2; 
		#8 tclk = 1; twr = 0; taddr = 2; 
		#8 tclk = 0; twr = 0; taddr = 3; 
		#8 tclk = 1; twr = 0; taddr = 3; 
		#8 tclk = 0; twr = 0; taddr = 4; 
		#8 tclk = 1; twr = 0; taddr = 4;
		#8 tclk = 0; twr = 0; taddr = 0;

	end
      
endmodule

