//
//		addsubbyp1as.v - 1-bit adder/subtracter - registered and unregistered
//
//					(C) Copyright 2004-2008 John B. Stephensen
//
// This Verilog source file and all its derivatives are licensed only for
// personal non-profit educational use in the Amateur Radio Service and
// the license is not transferrable. The information is provided as-is for
// experimental purposes and the author does not warranty its freedom
// from defects or its suitability for any specific application.
//
// This implementation uses a LUT2 as a half adder (XOR) followed by MUXCY to
// implement ripple carry and XORCY to complete a full adder.
//
//		I3 I2	I1	I0	O
//		0	0	0	0	1	SUB
//		0	0	0	1	0
//		0	0	1	0	0	9
//		0	0	1	1	1
//		0	1	0	0	0	ADD
//		0	1	0	1	1
//		0	1	1	0	1	6
//		0	1	1	1	0
//		1	0	0	0	1	SUB
//		1	0	0	1	1
//		1	0	1	0	0	3
//		1	0	1	1	0
//		1	1	0	0	0	ADD
//		1	1	0	1	0
//		1	1	1	0	1	C
//		1	1	1	1	1
//
module addsubbyp1as(
	 input ADD,
	 input BYP,
    input A,
    input B,
    input CI,
    output CO,
	 output Y,
    output Q,
	 input CLK,
	 input CE,
	 input RST
    );
// internal signals
wire h;	// half-adder output
// half adder
LUT4_L #(.INIT(16'hC369)) ha (.I0(A), .I1(B), .I2(ADD), .I3(BYP), .LO(h));
MUXCY_L cp (.CI(CI), .DI(A), .S(h), .LO(CO));
XORCY fa (.CI(CI), .LI(h), .O(Y));
FDCE ff (.D(Y), .C(CLK), .CE(CE), .CLR(RST), .Q(Q));
endmodule
