//
//			addsubincdec1a.v - 1-bit adder/subtracter/incrementer/decrementer
//
//					(C) Copyright 2007-2008 John B. Stephensen
//
// This Verilog source file and all its derivatives are licensed only for
// personal non-profit educational use in the Amateur Radio Service and
// the license is not transferrable. The information is provided as-is for
// experimental purposes and the author does not warranty its freedom
// from defects or its suitability for any specific application.
//
// This implementation uses a LUT2 as a half adder (XOR) followed by MUXCY to
// implement ripple carry and XORCY to complete a full adder. The output can
// also be set to zero or one.
//
//		I3 I2	I1	I0	O
//		0	0	0	0	1	SUB
//		0	0	0	1	0
//		0	0	1	0	0
//		0	0	1	1	1
//		0	1	0	0	0	ADD
//		0	1	0	1	1
//		0	1	1	0	1
//		0	1	1	1	0
//		1	0	0	0	0	INC (B=0)
//		1	0	0	1	1
//		1	0	1	0	0
//		1	0	1	1	1
//		1	1	0	0	1	DEC (B=1)
//		1	1	0	1	0
//		1	1	1	0	1
//		1	1	1	1	0
//
// The carry input is 1 for a subtract or increment and 0 for an add or decrement.
// The carry input may also reflect the carry output from a previous operation.
// The increment or decrement may be disabled by inverting the carry input.
//
module addsubincdec1a(
	 input [1:0] OP,
    input A,
    input B,
    input CI,
    output CO,
    output Y
    );
// internal signals
wire h;	// half-adder output
// half adder and se/reset logic
LUT4_L #(.INIT(16'h5A69)) ha (.I0(A), .I1(B), .I2(OP[0]), .I3(OP[1]), .LO(h));
// carry logic - selects CI when S = 1
MUXCY_L cp (.CI(CI), .DI(A), .S(h), .LO(CO));
// second half-adder
XORCY fa (.CI(CI), .LI(h), .O(Y));
endmodule
